////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
// Postgre SQL delphi unit v1.0
// =============================================================================
// Programmed by rageworx@gmail.com
//               http://rageworx.tistory.com
//               October 2010
//
// original code by Tilo Ermlich
//
////////////////////////////////////////////////////////////////////////////////
//
// Licenced under MOZILLA PUBLIC LICENSE Vers. 1.1, see licence.txt
// Tilo Ermlich - www.netzpol.de - 1st October 2007
//
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
unit UnitPostgreSQLclient;

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Grids, StrUtils, libpq_fe;

type
    TPSQLArray  = array of string;
    TPSQLMatrix = array of array of string;

    TPostgreSQL = class(TObject)
        private
            DBHost,
            DBUser,
            DBPass,
            DBName              : string;
            pg_ConnParms        : string;
            nRow                : integer;
            strError    : String;

        private // tools.
            function Split(const fText: String; const fSep: Char; fTrim: Boolean=false; fQuotes: Boolean=false):TStringList;
            function sar(sSrc, sLookFor, sReplaceWith: string ): string;
            function date_max(datum:String): String;
            function WriteLog(LogString: String): Integer;
            function uncontrol(var s:string):boolean;

            procedure errorf(const TheFormat: string;const Args:array of const); overload;
            procedure errorf(const TheFormat: string); overload;

            function  parse_metaid(sSrc: string ): string;
            procedure db_errorf(const TheFormat: string; const Args:array of const; SQLString: string);

        public
            constructor Create(AOwner : TComponent);

        public
            procedure sql_updateConnectParam;

        public
            function  sql_IfNull:String;
            function  sql_Limit(offset,rows: Integer):String;
            function  sql_Round(field:String;d:Integer):String;
            function  sql_Where:String;
            function  sql_Like:String;
            function  sql_SubDate(datestr, intervall: String): String;
            function  sql_Intervall(intervall: String): String;
            function  sql_Numsplit(zahl, flag:String):String;
            function  sql_DateFormat(column, format: String):String;
            function  sql_Path(path: String): String;
            function  sql_Now(): String;
            function  sql_Number(field:String;d:Integer):String;
            function  sql_NrConcat(field1, field2: String):String;
            function  sql_array_command(SQLArray: TPSQLArray):boolean;
            function  sql_command(SQLString: string):boolean;
            function  sql_select_row(SQLString: string; var myarray: TPSQLArray):boolean;
            function  sql_select_col(SQLString: string; var myarray: TPSQLArray):boolean;
            function  sql_select_sg(SQLString: string; var SQLGrid: TStringGrid; myarray: TPSQLArray):boolean;
            function  sql_select_mh_sg(SQLString: string; var SQLGrid: TStringGrid; limit: string='0'):boolean;
            function  sql_select_oh_sg(SQLString: string; var SQLGrid: TStringGrid):boolean;
            function  sql_arrayselect_oh_sg(SQLArray: TPSQLArray; var SQLGrid: TStringGrid):boolean;
            function  sql_arrayselect_matrix(SQLArray: TPSQLArray; var sqlrow: integer; var sqlcol: integer; var matrix: TPSQLMatrix):boolean;
            function  sql_select_matrix(SQLString: string; var sqlrow: integer; var sqlcol: integer; var matrix: TPSQLMatrix):boolean;
            function  sql_fill_listbox(SQLString: String; list_box : TListBox) : Boolean;
            function  sql_count(SQLString: String) : Integer;
            function  sql_parseout(sSrc: string ): string;

        public
            property propHost       : string read DBHost write DBHost;
            property propUser       : string read DBUser write DBUser;
            property propPassword   : string read DBPass write DBPass;
            property propName       : string read DBName write DBName;
            property lastErrorStr   : string read strError;

    end;

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
implementation

////////////////////////////////////////////////////////////////////////////////
const
    PG_CLIENTENCODING   = 'LATIN1';
    NETZPMESSAGE        = 'Demo - Message';
    LOGFILE             = 'UnitPostgreLog.txt';


////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
function TPostgreSQL.Split(const fText: String; const fSep: Char; fTrim: Boolean=false; fQuotes: Boolean=false):TStringList;
var
    vI: Integer;
    vBuffer: String;
    vOn: Boolean;
begin
    Result:=TStringList.Create;
    vBuffer:='';
    vOn:=true;
    for vI:=1 to Length(fText) do 
    begin
        if (fQuotes and(fText[vI]=fSep)and vOn)or(Not(fQuotes) and (fText[vI]=fSep)) then
        begin
            if fTrim then
                vBuffer:=Trim(vBuffer);

            if vBuffer='' then
                vBuffer:=fSep; // !!! sonst l?ft z.B. split(',**',',') auf einen Hammer...

            if vBuffer[1]=fSep then
                vBuffer:=Copy(vBuffer,2,Length(vBuffer));

            Result.Add(vBuffer);
            vBuffer:='';
        end;

        if fQuotes then
        begin
            if fText[vI]='"' then
            begin
                vOn:=Not(vOn);
                Continue;
            end;
            if (fText[vI]<>fSep)or((fText[vI]=fSep)and(vOn=false)) then
            vBuffer:=vBuffer+fText[vI];
        end else
            if fText[vI]<>fSep then
                vBuffer:=vBuffer+fText[vI];
    end;

    if vBuffer<>'' then
    begin
        if fTrim then
            vBuffer:=Trim(vBuffer);
            
        Result.Add(vBuffer);
    end;
end;

function TPostgreSQL.sar(sSrc, sLookFor, sReplaceWith: string ): string;
var
    nPos,
    nLenLookFor : integer;
begin
    nPos        := Pos( sLookFor, sSrc );
    nLenLookFor := Length( sLookFor );

    while(nPos > 0)do
    begin
        Delete( sSrc, nPos, nLenLookFor );
        Insert( sReplaceWith, sSrc, nPos );
        nPos := Pos( sLookFor, sSrc );
    end;

    Result := sSrc;
end;

function TPostgreSQL.date_max(datum:String): String;
begin
    if (datum = '') or (datum = '---') then
        result:='2111-11-11'
    else
        result := datum;
end;

function TPostgreSQL.WriteLog(LogString: String): Integer;
var
    f: TextFile;
begin
    {$IOChecks OFF}
        AssignFile(f, ExtractFilePath(ParamStr(0))+LOGFILE);

        if FileExists(ExtractFilePath(ParamStr(0))+LOGFILE) then
            Append(f)
        else
            Rewrite(f);

        Writeln(f, LogString);
        CloseFile(f);
        result := GetLastError();
    {$IOCHECKS ON}
end;

function TPostgreSQL.uncontrol(var s:string):boolean;
var
    i:integer;
begin
    i := 1;
    result := false;

    while i <= length(s) do
        if (ord(s[i])<32) or (ord(s[i])>126) then
        begin
            s := copy(s,1,i-1)+copy(s,i+1,10000);
            result := true;
        end else
            inc(i);
end;

procedure TPostgreSQL.errorf(const TheFormat: string;const Args:array of const);
begin
    strError := format(Theformat,args);
end;

procedure TPostgreSQL.errorf(const TheFormat: string);
begin
    errorf(TheFormat,[0]);
end;


////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
constructor TPostgreSQL.Create(AOwner : TComponent);
begin
    inherited Create();
end;

procedure TPostgreSQL.sql_updateConnectParam;
begin
    pg_ConnParms := 'host=''' + DBHost + ''' user=''' + DBUser + ''' dbname=''' + DBName + ''' password=''' + DBPass + '''';
end;

// #############################################################################
// myIfNull: encapsulates an if-null-decision,
//      i.e. '... sum(' + myIfNull + '(a.value + b.value,a.value)) ...'
// #############################################################################
function TPostgreSQL.sql_IfNull:String;
begin
    result:='coalesce';
end;


// #############################################################################
// sql_Limit: encapsulates a limit for result sets
// #############################################################################
function TPostgreSQL.sql_Limit(offset,rows: Integer):String;
begin
    result:='limit ' + inttostr(rows) + ' offset ' + inttostr(offset);
end;


// #############################################################################
// sql_Round: encapsulates rounding, i.e. 'select ' + sql_Round('value',2) + ' ...'
// #############################################################################
function TPostgreSQL.sql_Round(field:String;d:Integer):String;
begin
    result:='round(cast(' + field + ' as numeric),' + inttostr(d) + ')';
end;


// #############################################################################
// sql_Where: encapsulates an everytime matching where-condition, useful for building dynamic statements
// building dynymic statements, i.e. 'select * from address where ' + sql_Where + ' ...'
// #############################################################################
function TPostgreSQL.sql_Where:String;
begin
    result:='true';
end;


// #############################################################################
// sql_Like: encapsulates the like-clause,
// i.e. 'select * from address where name ' + sql_Like + ' ''%' + string + '%'' order by name;';
// #############################################################################
function TPostgreSQL.sql_Like:String;
begin
    result:='ilike';
end;


// #############################################################################
// sql_SubDate: encapsulates date substraction, i.e. 'select ' + sql_SubDate('2007-07-07', '30 DAY');
// #############################################################################
function TPostgreSQL.sql_SubDate(datestr, intervall: String): String;
begin
    //MySQL: date_sub("yyyy-mm-dd", interval 30 day):
    //MySQL: result:='DATE_SUB(''' + datestr + ''', INTERVALL ' + intervall + ')';
    //PostgreSQL: date('2007-07-07') - interval '30 days'
    result:= 'date(''' + datestr + ''') - interval ''' + intervall + '''';
end;


// #############################################################################
// sql_Intervall: encapsulates date substraction from now,
// i.e. 'select ... where (thedate >= ' + sql_Intervall('1 year') + ') and ...'
// #############################################################################
function TPostgreSQL.sql_Intervall(intervall: String): String;
begin
    //now() - interval 1 year:
    result:='now() - interval ''' + intervall + '''';
end;


// #############################################################################
// sql_Numsplit: function for separating a floating point by the point,
// sql_Numsplit('-12345.6789',v) = '-12345', sql_Numsplit('-12345.6789',n) = '6789'
// #############################################################################
function TPostgreSQL.sql_Numsplit(zahl, flag:String):String;
var
    azahl: TStringList;
begin
    if (flag = 'v') then
    begin
        azahl:=split(zahl,'.',true,false);
        if (azahl.count = 0) then
            result := '0'
        else
            result := azahl[0];
    end;

    if (flag = 'n') then
    begin
        azahl:=split(zahl,'.',true,false);
        if (azahl.count = 0) or (azahl.count = 1) then
            result := '0'
        else
            result := azahl[1];
    end;
end;


// #############################################################################
// sql_DateFormat: encapsulates date formating,
// i.e. 'select name, ' + sql_DateFormat('birthdate', '%d.%m.%Y') + ', ...'
// #############################################################################
function TPostgreSQL.sql_DateFormat(column, format: String):String;
begin
    //date_format(datum, "%d.%m.%Y"):
    //MySQL: result:='date_format(' + column + ', ''' + format + ''')';
    result:='to_char(' + column + ', ''DD.MM.YYYY'')';
    //result:='replace(to_char(' + column + ', ''DD.MM.YYYY''),''01.01.1970'',''00.00.0000'')';
end;


// #############################################################################
// sql_Path: encapsulates representation of directory paths,
// i.e. 'select * from documents where pfad = ' + sql_Path('\DocPlace\Products\') + ' ...'
// #############################################################################
function TPostgreSQL.sql_Path(path: String): String;
begin
    //AnsiReplaceStr(Main_Form.frame_formular.dokuvorl.Text, '\', '\\'):
    result:= 'E''' + AnsiReplaceStr(path, '\', '\\');
end;


// #############################################################################
// sql_Now: encapsulates the now-statement,
// i.e. 'update table set date = ' + sql_Now() + ' where ...'
// #############################################################################
function TPostgreSQL.sql_Now(): String;
begin
    //now():
    result:='now()';
end;


// #############################################################################
// sql_Number: encapsulates number formating,
// i.e. 'select product, ' + sql_Number('amount',3) + ' ...'
// #############################################################################
function TPostgreSQL.sql_Number(field:String;d:Integer):String;
var
    format: String;
begin
    //MySQL: replace(replace(replace(format(field,d),',','&t;'),'.',','),'&t;','.'): 1,004,234.22 -> 1.004.234,22
    //MySQL: result:= 'replace(replace(replace(format(' + field + ',' + d + '),'','',''&t;''),''.'','',''),''&t;'',''.'')';

    case d of
        1: format := '''FM999G999G999G990D099999''';
        2: format := '''FM999G999G999G990D009999''';
        3: format := '''FM999G999G999G990D000999''';
        4: format := '''FM999G999G999G990D000099''';
        5: format := '''FM999G999G999G990D000009''';
    end;

    result:='to_char(round(cast(' + field + ' as numeric),' + inttostr(d) + '), ' + format + ')';
end;


// #############################################################################
// sql_NrConcat: special function for concating numbers
// #############################################################################
function TPostgreSQL.sql_NrConcat(field1, field2: String):String;
begin
    //MySQL: concat(a.auftrgnr, "-", a.zusatznr):
    //result:='concat(' + field1 + ', ''-'',' + field2 + ')';
    result := field1 + '|| ''-'' ||' + field2;
end;


// #############################################################################
// db_errorf: special function from J.G. Owen for formating exception messages
// #############################################################################
procedure TPostgreSQL.db_errorf(const TheFormat: string; const Args:array of const; SQLString: string);
var s:string;
begin
    s := format(TheFormat,Args);
    uncontrol(s);           //they have \ns in there silly children.
    errorf('%s',[s]);       //don't let %s in messages blow-up format.

    {$IfDef SQLLOG}
    Clipboard.AsText :=SQLString;
    {$EndIf}
end;

// #############################################################################
// sql_parseout: special function substituting the tag --- back to dummy integer -1
// #############################################################################
function TPostgreSQL.parse_metaid(sSrc: string ): string;
var
    nPos: integer;
begin
    nPos := Pos( '---', sSrc );

    while(nPos > 0)do
    begin
        Delete( sSrc, nPos, 3 );
        Insert( '-1', sSrc, nPos );
        nPos := Pos( '---', sSrc );
    end;
    
    Result := sSrc;
end;

// #############################################################################
// sql_array_command: sql-procedure for sending multitple sql-statements to the database
// while the connection is hold
// #############################################################################
function TPostgreSQL.sql_array_command(SQLArray: TPSQLArray) : boolean;
//Parsing for ', etc. must be implemented in program code
var
    conn:p_PGconn;
    res : p_PGresult;
    test: boolean;
    exce: string;
    k: integer;
label
    ENDNESS;
begin
    k:=1;
    test:=false;
    res:=nil;
    conn := PQconnectdb(PCHAR(pg_ConnParms));
    PQsetClientEncoding(conn, pChar(pg_ClientEncoding));
    exce := 'Connection to database failed';

    if (PQstatus(conn)<>CONNECTION_OK) then
        goto ENDNESS;

    try
        for k := 1 to length(SQLArray) do
        begin
            SQLArray[k-1]:=sar(SQLArray[k-1],'-- Selection --','-1');
            SQLArray[k-1]:=sar(SQLArray[k-1],'-- No Result --','-1');
            {$IfDef SQLLOG}
            writeLog('sql_array_command: ' + SQLArray[k-1]);
            Clipboard.AsText :='sql_array_command: ' + SQLArray[k-1];
            {$EndIf}

            exce := 'Command ''' + SQLArray[k-1] + ''' failed';
            PQclear(res);
            res := PQexec(conn,pChar(SQLArray[k-1]));
        end;

        if (PQresultStatus(res) = PGRES_COMMAND_OK) then
        begin
            test:=true;
        end;
    finally
    end;


    ENDNESS:
    if not(test) then
        db_errorf('%s: %s',[exce,PQerrorMessage(conn)],SQLArray[k-1]);

    if (res <> nil) then
        PQclear(res);
        
    PQfinish(conn);

    result := test;
end;


// #############################################################################
// sql_command: the basic sql-procedure for excecution of database commands
// #############################################################################
function TPostgreSQL.sql_command(SQLString: string):boolean;
//Parsing for ', etc. must be implemented in program code
var
    conn:p_PGconn;
    res : p_PGresult;
    test: boolean;
    exce: string;
label
    ENDNESS;
begin
    SQLString:=sar(SQLString,'-- Selection --','-1');
    //SQLString:=sar(SQLString,'-- No Result --','-1');

    {$IfDef SQLLOG}
    writeLog('sql_command: ' + SQLString);
    Clipboard.AsText :='sql_command: ' + SQLString;
    {$EndIf}

    test:=false;
    res:=nil;
    conn := PQconnectdb(PCHAR(pg_ConnParms));
    PQsetClientEncoding(conn, pChar(pg_ClientEncoding));
    exce := 'Connection to database failed';

    if (PQstatus(conn)<>CONNECTION_OK) then
        goto ENDNESS;

    try
        exce := 'Command ''' + SQLString + ''' failed';
        PQclear(res);
        res := PQexec(conn,pChar(SQLString));

        if (PQresultStatus(res) = PGRES_COMMAND_OK)  then
        begin
            test:=true;
        end;

    finally
    end;


    ENDNESS:
    if not(test) then
        db_errorf('%s: %s',[exce,PQerrorMessage(conn)],SQLString);

    if (res <> nil) then
        PQclear(res);
        
    PQfinish(conn);

    result := test;
end;


// #############################################################################
// sql_select_row: the basic sql-procedure for selecting a row and loading the
// result into a dynamic array called by reference
// #############################################################################
function TPostgreSQL.sql_select_row(SQLString: string; var myarray: TPSQLArray):boolean;
// Parsing for ' implemented here
var
    conn:p_PGconn;
    res : p_PGresult;
    test: boolean;
    exce: string;
    j, numcol:integer;
label
    ENDNESS;
begin
    {$IfDef SQLLOG}
    writeLog('sql_select_row: ' + SQLString);
    Clipboard.AsText :='sql_select_row: ' + SQLString;
    {$EndIf}

    test:=false;
    res:=nil;
    conn := PQconnectdb(PCHAR(pg_ConnParms));
    PQsetClientEncoding(conn, pChar(pg_ClientEncoding));
    exce := 'Connection to database failed';

    if (PQstatus(conn)<>CONNECTION_OK) then
        goto ENDNESS;

    SQLString:=parse_metaid(SQLString);

    try
        exce := 'Command ''' + SQLString + ''' failed';
        PQclear(res);
        res := PQexec(conn,pChar(SQLString));

        if (PQresultStatus(res) = PGRES_TUPLES_OK) then
        begin
            test:=true;
            if (res <> nil) then
            begin
                numcol:= PQnfields(res);
                SetLength(myarray,numcol + 1);

                for j := 0 to numcol - 1 do
                begin
                    if PQntuples(res) <> 0 then
                        myarray[j]:= sql_parseout(PQgetvalue(res,0,j))
                    else
                        myarray[j]:= '---';
                end;
            end
            else
            begin
                SetLength(myarray, 100);
                
                for j := 0 to 99 do
                    myarray[j] := '-- No Result --'
            end;
        end;
    finally
    end;

    ENDNESS:
    if not(test) then
        db_errorf('%s: %s',[exce,PQerrorMessage(conn)],SQLString);

    if (res <> nil) then
        PQclear(res);
        
    PQfinish(conn);

    result := test;
end;


// #############################################################################
// sql_select_col: the basic sql-procedure for selecting a column and loading
// the result into a dynamic array called by reference
// #############################################################################
function TPostgreSQL.sql_select_col(SQLString: string; var myarray: TPSQLArray):boolean;
// Parsing for ' implemented here
var
    conn:p_PGconn;
    res : p_PGresult;
    test: boolean;
    exce: string;
    i, numrow:integer;
label
    ENDNESS;
begin
    {$IfDef SQLLOG}
    writeLog('sql_select_col: ' + SQLString);  
    Clipboard.AsText :='sql_select_col: ' + SQLString;
    {$EndIf}
  
    test:=false;
    res:=nil;
    conn := PQconnectdb(PCHAR(pg_ConnParms));
    PQsetClientEncoding(conn, pChar(pg_ClientEncoding));
    exce := 'Connection to database failed';
    
    if (PQstatus(conn)<>CONNECTION_OK) then
        goto ENDNESS;

    try
        exce := 'Command ''' + SQLString + ''' failed';
        PQclear(res);
        res := PQexec(conn,pChar(SQLString));

        if (PQresultStatus(res) = PGRES_TUPLES_OK) then
        begin
            test:=true;

            if (res <> nil) then
            begin
                numrow:= PQntuples(res) + 1;
                SetLength(myarray,numrow);

                if (numrow > 1) then
                    for i := 1 to numrow - 1 do
                        myarray[i]:= sql_parseout(PQgetvalue(res,i - 1,0))
                else
                begin
                    SetLength(myarray, 2);
                    myarray[1]:= '---';
                end
            end
            else
            begin
                SetLength(myarray, 2);
                myarray[1] := '-- No Result --'
            end;
        end;

    finally
    end;

    ENDNESS:
    if not(test) then
        db_errorf('%s: %s',[exce,PQerrorMessage(conn)],SQLString);

    if (res<>nil) then
        PQclear(res);
        
    PQfinish(conn);

    result := test;
end;


// #############################################################################
// sql_select_sg: sql-procedure for fetching a result set and loading it into
// a string grid called by reference, i.e. useful for string grids WITHOUT column headers
// #############################################################################
function TPostgreSQL.sql_select_sg(SQLString: string; var SQLGrid: TStringGrid; myarray: TPSQLArray):boolean;
// Parsing for ' implemented here
var
    conn:p_PGconn;
    res : p_PGresult;
    test: boolean;
    exce: string;
    i,j,
    numcol,
    numrow:integer;
label
    ENDNESS;
begin
    {$IfDef SQLLOG}
    writeLog('sql_select_sg: ' + SQLString);
    Clipboard.AsText :='sql_select_sg: ' + SQLString;
    {$EndIf}

    test:=false;
    res:=nil;
    conn := PQconnectdb(PCHAR(pg_ConnParms));
    PQsetClientEncoding(conn, pChar(pg_ClientEncoding));
    exce := 'Connection to database failed';

    if (PQstatus(conn)<>CONNECTION_OK) then
        goto ENDNESS;

    try
        exce := 'Command ''' + SQLString + ''' failed';
        PQclear(res);
        res := PQexec(conn,pChar(SQLString));

        if (PQresultStatus(res) = PGRES_TUPLES_OK) then
        begin
            test:=true;
            if (res <> nil) then
            begin
                numcol:= PQnfields(res);
                numrow:= PQntuples(res) + 1;

                SQLGrid.ColCount:= numcol;
                SQLGrid.RowCount:= numrow;

                for j := 0 to numcol -1 do
                    SQLGrid.Cells[j, 0] := sql_parseout(myarray[j]);

                for i := 1 to numrow do
                    for j := 0 to numcol -1 do
                        SQLGrid.Cells[j, i]:= sql_parseout(PQgetvalue(res,i - 1,j));

                if numrow > 1 then
                begin
                    SQLGrid.FixedRows:=1;
                    SQLGrid.FixedColor:= clBtnFace;
                end;

                if numrow = 1 then
                begin
                    SQLGrid.RowCount:=2;
                    SQLGrid.FixedRows:=1;

                    for j := 0 to numcol - 1 do
                        SQLGrid.Cells[j, 1]:= '---';
                end;
            end;
        end;
    finally
    end;

    ENDNESS:
    if not(test) then
        db_errorf('%s: %s',[exce,PQerrorMessage(conn)],SQLString);

    if (res<>nil) then
        PQclear(res);

    PQfinish(conn);
    result := test;
end;


// #############################################################################
// sql_select_mh_sg: sql-procedure for fetching a result set and loading it into
// a string grid called by reference, i.e. useful for string grids WITH column headers
// which should be filled with the first row of the result set
// #############################################################################
function TPostgreSQL.sql_select_mh_sg(SQLString: string; var SQLGrid: TStringGrid; limit: string='0'):boolean;
// Parsing for ' implemented here
var
    conn:p_PGconn;
    res : p_PGresult;
    test: boolean;
    exce: string;
    i,j, numcol, numrow:integer;
label
    ENDNESS;
begin
    if (limit <> '0') then
        SQLString := SQLString + ' limit ' + limit + ';';

    {$IfDef SQLLOG}
    writeLog('sql_select_mh_sg: ' + SQLString);
    Clipboard.AsText :='sql_select_mh_sg: ' + SQLString;
    {$EndIf}

    test:=false;
    res := nil;
    conn := PQconnectdb(PCHAR(pg_ConnParms));
    PQsetClientEncoding(conn, pChar(pg_ClientEncoding));
    exce := 'Connection to database failed';

    if (PQstatus(conn)<>CONNECTION_OK) then
        goto ENDNESS;

    try
        exce := 'Command ''' + SQLString + ''' failed';
        PQclear(res);
        res := PQexec(conn,pChar(SQLString));

        if (PQresultStatus(res) = PGRES_TUPLES_OK) then
        begin
            test:=true;
            if res <> nil then
            begin
                numcol:= PQnfields(res);
                numrow:= PQntuples(res);
                SQLGrid.ColCount:= numcol;
                SQLGrid.RowCount:= numrow;

                if numrow = 0 then
                for j := 0 to numcol - 1 do
                    SQLGrid.Cells[j, 0]:= '---';

                for i := 0 to numrow - 1 do
                    for j := 0 to numcol - 1 do
                        SQLGrid.Cells[j, i]:= sql_parseout(PQgetvalue(res,i,j));
            end;
        end;

    finally
    end;

    ENDNESS:
    if not(test) then
        db_errorf('%s: %s',[exce,PQerrorMessage(conn)],SQLString);

    if (res <> nil) then
        PQclear(res);
        
    PQfinish(conn);
    result := test;
end;


// #############################################################################
// sql_select_oh_sg: sql-procedure for fetching a result set and loading it into
// a string grid called by reference, i.e. useful for string grids WITH column headers
// which should NOT be filled with the first row of the result set
// #############################################################################
function TPostgreSQL.sql_select_oh_sg(SQLString: string; var SQLGrid: TStringGrid):boolean;
// Parsing for ' implemented here
var
    conn:p_PGconn;
    res : p_PGresult;
    test: boolean;
    exce: string;
    i,j, numcol, numrow:integer;
label
    ENDNESS;
begin
    {$IfDef SQLLOG}
    writeLog('sql_select_oh_sg: ' + SQLString);
    Clipboard.AsText :='sql_select_oh_sg: ' + SQLString;
    {$EndIf}

    test:=false;
    res := nil;
    conn := PQconnectdb(PCHAR(pg_ConnParms));
    PQsetClientEncoding(conn, pChar(pg_ClientEncoding));
    exce := 'Connection to database failed';

    if (PQstatus(conn)<>CONNECTION_OK) then
        goto ENDNESS;

    try
        exce := 'Command ''' + SQLString + ''' failed';
        PQclear(res);
        res := PQexec(conn,pChar(SQLString));

        if (PQresultStatus(res) = PGRES_TUPLES_OK) then
        begin
            test:=true;

            if res <> nil then
            begin
                numcol:= PQnfields(res);
                numrow:= PQntuples(res);
                SQLGrid.ColCount:= numcol;

                if numrow = 0 then
                begin
                    SQLGrid.RowCount:= numrow + 2;
                    for j := 0 to numcol - 1 do
                    begin
                        SQLGrid.Cells[j, 1]:= '---';
                    end;
                end
                else
                begin
                    SQLGrid.RowCount:= numrow + 1;
                    for i := 0 to numrow - 1 do
                    begin
                        for j := 0 to numcol -1 do
                            SQLGrid.Cells[j, (i + 1)]:= sql_parseout(PQgetvalue(res,i,j));
                    end;
                end;
            end;
        end;

    finally
    end;

    ENDNESS:
    if not test then
        db_errorf('%s: %s',[exce,PQerrorMessage(conn)],SQLString);

    if res <> nil then
        PQclear(res);
        
    PQfinish(conn);
    result := test;
end;


// #############################################################################
// sql_arrayselect_oh_sg: sql-procedure for fetching a result set by multiple
// sql-commands while the connection is hold and then loading it into a string grid
// called by reference, i.e. useful with temporary tables and for string grids
// WITH column headers which should NOT be filled with the first row of the result set
// #############################################################################
function TPostgreSQL.sql_arrayselect_oh_sg(SQLArray: TPSQLArray; var SQLGrid: TStringGrid):boolean;
// Parsing for ' implemented here
var
    conn:p_PGconn;
    res : p_PGresult;
    test: boolean;
    exce: string;
    i, j, k, numcol, numrow:integer;
label
    ENDNESS;
begin
    k := 1;
    test:=false;
    res := nil;
    conn := PQconnectdb(PCHAR(pg_ConnParms));
    PQsetClientEncoding(conn, pChar(pg_ClientEncoding));
    exce := 'Connection to database failed';

    if (PQstatus(conn)<>CONNECTION_OK) then
        goto ENDNESS;

    try
        for k := 1 to length(SQLArray) do
        begin
            {$IfDef SQLLOG}
            writeLog('sql_arrayselect_matrix_oh_sg: ' + SQLArray[k-1]);
            Clipboard.AsText :='sql_arrayselect_matrix_oh_sg: ' + SQLArray[k-1];
            {$EndIf}

            exce := 'Command ''' + SQLArray[k-1] + ''' failed';
            PQclear(res);
            res := PQexec(conn,pChar(SQLArray[k-1]));

            //if (PQresultStatus(res) <> PGRES_TUPLES_OK) then goto ENDNESS;
        end;

        if (PQresultStatus(res) = PGRES_TUPLES_OK) then
        begin
        test:=true;
            if res <> nil then
            begin
                numcol:= PQnfields(res);
                numrow:= PQntuples(res);

                SQLGrid.ColCount:= numcol;


                if numrow = 0 then
                begin
                    SQLGrid.RowCount:= numrow + 2;
                    for j := 0 to numcol - 1 do
                        SQLGrid.Cells[j, 1]:= '---';
                end
                else
                begin
                    SQLGrid.RowCount:= numrow + 1;
                    for i := 0 to numrow - 1 do
                        for j := 0 to numcol - 1 do
                            SQLGrid.Cells[j, (i + 1)]:= sql_parseout(PQgetvalue(res,i,j));
                end;
            end;
        end;

    finally
    end;

    ENDNESS:
    if not test then
        db_errorf('%s: %s',[exce,PQerrorMessage(conn)],SQLArray[k-1]);
        
    if res <> nil then
        PQclear(res);

    PQfinish(conn);
    result := test;
end;


// #############################################################################
// sql_arrayselect_matrix: sql-procedure for fetching a result set by multiple
// sql-commands while the connection is hold and then loading it into a dynamic matrix
// called by reference, i.e. useful with temporary tables and for result sets which should
// by used in another way (MS Excel export)
// #############################################################################
function TPostgreSQL.sql_arrayselect_matrix(SQLArray: TPSQLArray; var sqlrow: integer; var sqlcol: integer; var matrix: TPSQLMatrix):boolean;
// Parsing for ' implemented here
var
    conn:p_PGconn;
    res : p_PGresult;
    test: boolean;
    exce: string;
    i, j, k, numcol, numrow:integer;
label
    ENDNESS;
begin
    k:=1;
    test:=false;
    res := nil;
    conn := PQconnectdb(PCHAR(pg_ConnParms));
    PQsetClientEncoding(conn, pChar(pg_ClientEncoding));
    exce := 'Connection to database failed';

    if (PQstatus(conn)<>CONNECTION_OK) then
        goto ENDNESS;

    try
        for k := 1 to length(SQLArray) do
        begin
            {$IfDef SQLLOG}
            writeLog('sql_arrayselect_matrix: ' + SQLArray[k-1]);
            Clipboard.AsText :='sql_arrayselect_matrix: ' + SQLArray[k-1];
            {$EndIf}

            exce := 'Command ''' + SQLArray[k-1] + ''' failed';
            PQclear(res);
            res := PQexec(conn,pChar(SQLArray[k-1]));
        end;

        if (PQresultStatus(res) = PGRES_TUPLES_OK) then
        begin
            test:=true;
            if res <> nil then
            begin
                numcol:= PQnfields(res);
                numrow:= PQntuples(res);

                sqlcol:= numcol;
                sqlrow:= numrow;
                SetLength(matrix,numrow,numcol);

                for i := 0 to numrow - 1 do
                for j := 0 to numcol - 1 do matrix[i,j]:= sql_parseout(PQgetvalue(res,i,j));
            end;
        end;

    finally
    end;

    ENDNESS:
    if not test then
        db_errorf('%s: %s',[exce,PQerrorMessage(conn)],SQLArray[k-1]);

    if res <> nil then
        PQclear(res);

    PQfinish(conn);
    result := test;
end;


// #############################################################################
// sql_select_matrix: sql-procedure for fetching a result set and loading it into
// a dynamic matrix called by reference, i.e. useful for result sets which should
// by used in another way (MS Excel export)
// #############################################################################
function TPostgreSQL.sql_select_matrix(SQLString: string; var sqlrow: integer; var sqlcol: integer; var matrix: TPSQLMatrix):boolean;
// Parsing for ' implemented here
var
    conn:p_PGconn;
    res : p_PGresult;
    test: boolean;
    exce: string;
    i,j, numcol, numrow:integer;
label
    ENDNESS;
begin
    {$IfDef SQLLOG}
    writeLog('sql_select_matrix: ' + SQLString);
    Clipboard.AsText :='sql_select_matrix: ' + SQLString;
    {$EndIf}

    test:=false;
    res := nil;
    conn := PQconnectdb(PCHAR(pg_ConnParms));
    PQsetClientEncoding(conn, pChar(pg_ClientEncoding));
    exce := 'Connection to database failed';

    if (PQstatus(conn)<>CONNECTION_OK) then
        goto ENDNESS;

    try
        exce := 'Command ''' + SQLString + ''' failed';
        PQclear(res);
        res := PQexec(conn,pChar(SQLString));

        if (PQresultStatus(res) = PGRES_TUPLES_OK) then
        begin
            test:=true;
            if (res <> nil) then
            begin
                numcol:= PQnfields(res);
                numrow:= PQntuples(res);

                sqlcol:= numcol;
                sqlrow:= numrow;
                SetLength(matrix,numrow,numcol);

                for i := 0 to numrow - 1 do
                    for j := 0 to numcol - 1 do
                        matrix[i,j]:= sql_parseout(PQgetvalue(res,i,j));
            end;
        end;

    finally
    end;

    ENDNESS:
    if not(test) then
        db_errorf('%s: %s',[exce,PQerrorMessage(conn)],SQLString);

    if (res <> nil) then
        PQclear(res);
        
    PQfinish(conn);
    result := test;
end;


// #############################################################################
// sql_fill_listbox: sql-procedure, fills a ListBox with results of a SQL statement,
// gives back true for filled result set otherwise false
// #############################################################################
function TPostgreSQL.sql_fill_listbox(SQLString: String; list_box : TListBox) : Boolean;
// Parsing for ' implemented here
var
    conn:p_PGconn;
    res : p_PGresult;
    test: boolean;
    exce: string;
    i, numrow:integer;
label
    ENDNESS;
begin
    {$IfDef SQLLOG}
    writeLog('sql_fill_listbox, ' + list_box.Name + ': ' + SQLString);
    Clipboard.AsText :='sql_fill_listbox, ' + list_box.Name + ': ' + SQLString;
    {$EndIf}

    sql_fill_listbox := false;
    test:=false;
    res:=nil;
    conn := PQconnectdb(PCHAR(pg_ConnParms));
    PQsetClientEncoding(conn, pChar(pg_ClientEncoding));
    exce := 'Connection to database failed';

    if (PQstatus(conn)<>CONNECTION_OK) then
        goto ENDNESS;

    try
        exce := 'Command ''' + SQLString + ''' failed';
        PQclear(res);
        res := PQexec(conn,pChar(SQLString));

        if (PQresultStatus(res) = PGRES_TUPLES_OK) then
        begin
            test:=true;
            if res <> nil then
            begin
                list_box.Clear;

                numrow:= PQntuples(res) + 1;
                for i := 1 to numrow -1 do
                    list_box.Items.Add(sql_parseout(PQgetvalue(res,i,0)));
                sql_fill_listbox := true;
            end;
        end;
    finally
    end;

    ENDNESS:
    if not(test) then
        db_errorf('%s: %s',[exce,PQerrorMessage(conn)],SQLString);

    if (res<>nil) then
        PQclear(res);
        
    PQfinish(conn);
end;


// #############################################################################
// sql_count: sql-procedure, counts the number of results for a given SQL string
// #############################################################################
function TPostgreSQL.sql_count(SQLString: String) : Integer;
// Parsing for ' implemented here
var
    conn:p_PGconn;
    res : p_PGresult;
    test: boolean;
    exce: string;
    i, numrow, count:integer;
label
    ENDNESS;
begin
    {$IfDef SQLLOG}
    writeLog('sql_count: ' + SQLString);
    Clipboard.AsText :='sql_count: ' + SQLString;
    {$EndIf}

    count:= 0;

    test:=false;
    res:=nil;
    conn := PQconnectdb(PCHAR(pg_ConnParms));
    PQsetClientEncoding(conn, pChar(pg_ClientEncoding));
    exce := 'Connection to database failed';

    if (PQstatus(conn)<>CONNECTION_OK) then
        goto ENDNESS;

    try
        exce := 'Command ''' + SQLString + ''' failed';
        PQclear(res);
        res := PQexec(conn,pChar(SQLString));

        if (PQresultStatus(res) = PGRES_TUPLES_OK) then
        begin
            test:=true;
            if res <> nil then
            begin
                count:= PQntuples(res);
            end;
        end;
    finally
    end;

    ENDNESS:
    if not(test) then
        db_errorf('%s: %s',[exce,PQerrorMessage(conn)],SQLString);

    if (res<>nil) then
        PQclear(res);
        
    PQfinish(conn);
    sql_count := count;
end;


// #############################################################################
// sql_parseout: special function which substitutes the tag $apos; back to '
// #############################################################################
function TPostgreSQL.sql_parseout(sSrc: string ): string;
begin
  Result := sar(sar(sSrc,
           '11.11.2111','00.00.0000'),
           '$apos;','''');
end;


end.
